@@ -20,6 +20,8 @@ use Carp;
 use Test::More;
 #use Text::ParseWords qw(shellwords);
 
+use Build::Hopen::Base;
+
 # Definitions from this file
 use constant {
     true => !!1,
@@ -47,8 +49,8 @@ sub import {
     # Re-export pragmas and packages
     feature->import::into($target, qw(:5.14));
     Carp->import::into($target, qw(carp croak confess));
-    foreach my $pragma (qw(strict warnings Data::Dumper Test::More)) {
-        ${pragma}->import::into($target);
+    foreach (qw(strict warnings Data::Dumper Test::More Build::Hopen::Base)) {
+        $_->import::into($target);
     };
 
 } #import()
